package com.ibm.iru.db2express9_1lnx;

/**
 * @(#)DB2Express9_1LnxMain.java
 *
 * Licensed Materials - Property of IBM
 *
 * 5724-J10
 *
 * (C) Copyright IBM Corp. 2007
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *
 * This is sample code made available for use in accordance with terms set forth
 * in the license agreement document for the IBM Express Runtime.
 */

import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.PrintWriter;

import com.ibm.iru.common.LicenseHandler;
import com.ibm.iru.message.NLSKeys;
import com.ibm.jsdt.support.SupportLinuxBase;
import com.ibm.jsdt.support.SupportLinuxHelper;
import com.ibm.jsdt.support.installedproduct.DB2InstalledProduct;

/**
 * 
 * This class launches the DB2 Express 9.1 installation.
 *
 */
public class DB2Express9_1LnxMain extends SupportLinuxBase
{
  private static final String copyright0 = "Licensed Materials - Property of IBM";
  private static final String copyright1 = "5724-J10";
  private static final String copyright2 = "(C) Copyright IBM Corp. 2007 All Rights Reserved.";
  private static final String copyright3 = "US Government Users Restricted Rights - Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.";

  private static final int RC_SUCCESS_WITH_WARNINGS = 1;

  private String ivMediaFileName = null;
  private String ivNlPackFileName = null; //DB2 package for language files
  private SupportLinuxHelper ivHelper;
  private String ivInstallDir = null;
  private String[] ivLangs = { "BR", "CN", "CZ", "DE", "ES", "FR", "IT", "JP", "KR", "PL", "RU", "TW" };
  private String ivLicenseFileName = "db2exp_o.lic"; //license file to register after installation

  /**
   * The init initializes instance variables
   * @param  args[0] - The name of the response file
   * @param  args[1] - The name of the image tar file
   * @param  args[2] - The name of the nlpack tar file
   * 
   */
  private int init(String args[])
  {
    int rc = SUCCESS;

    // Set initial data
    setMainResources("com.ibm.iru.message.MessagesNLS");
    ivHelper = getLinuxHelper();
    int no_Of_Expected_args = 3;

    if (no_Of_Expected_args == args.length)
    {
      setResponseFileName(args[0].trim());
      ivMediaFileName = args[1].trim();
      ivNlPackFileName = args[2].trim();
      //installation directory
      setKey("FILE");
      ivInstallDir = ivHelper.trimDoubleQuotes(ivHelper.getResponseFileValue(this));
    }
    else
    {
      //an incorrect number of arguments were passed log the default
      // values that were used.
      String[] replacementText = { "DB2Express9_1LnxMain", String.valueOf(no_Of_Expected_args),
          String.valueOf(args.length) };
      setMessage(getResourceString(NLSKeys.BAD_NUMBER_PGM_ARGS, replacementText));
      ivHelper.log(this);
      rc = FAILURE;
    }

    return rc;
  }

  /**
   * Prepare the system for an install. Set/append variables values in response file:
   * <li>1. Check if a DAS user name is provided. If so, check that there is no DAS user already
   * in the system. There can only be one DAS user per system. If a DAS user name is specified,
   * and a DAS user already exists in the system, post a warning and continue install (ignoring 
   * specified DAS user info). If there is not a DAS user in the system, update response file with DAS info.
   * <li>2. Check if DAS user name already exists as instance user. If so, post a warning and continue install,
   * ignoring specified DAS user info.
   * <li>3. Update response file with languages selected by the user.
   * 4. If the user specifies a different port to use, then determine if in fact this port is valid.
   * If in fact it is valid than a service is established with that port number and the name of that 
   * service will be the instance name.
   * @return SUCCESS if the updates to the response file were successful, FAILURE otherwise.
   */
  private int prepareForInstall()
  {
    int rc = SUCCESS;

    //check if DAS user name and password is specified, if a DAS user already exists in the system.
    String dasUserName = getProperty("dasUserName");
    String dasUserPass = ivHelper.getPasswordValue("dasUserPassword");

    if (dasUserName != null && !dasUserName.equals("") && dasUserPass != null && !dasUserPass.equals(""))
    {
      String existingDASUser = getDASUser();
      if (existingDASUser != null && !existingDASUser.equals(""))
      {
        //a DAS user already exists. Log warning indicating that the DAS user name 
        //and password provided will not be used.
        setMessage(getResourceString(NLSKeys.DB2_DAS_USER_ALREADY_EXISTS, existingDASUser));
        ivHelper.postWarningMessageToDeployerUI(this);

      }
      else if (isDASUserInstanceUser(dasUserName))
      {
        //DAS user name provided already exists in the system as a DB2 instance user name.
        //Log warning indicating that the DAS user cannot be the same as the instance user, and 
        //continue install without using this information.
        setMessage(getResourceString(NLSKeys.DB2_DAS_USER__EXISTS_AS_INSTANCE, dasUserName));
        ivHelper.postWarningMessageToDeployerUI(this);
      }
      else
      {
        setResponseFileValue("DAS_USERNAME", dasUserName);
        setResponseFileValue("DAS_PASSWORD", dasUserPass);
        setResponseFileValue("DAS_GROUP_NAME", getProperty("dasGroup"));
        setResponseFileValue("DAS_HOME_DIRECTORY", getProperty("dasHomeDir"));
      }
    }

    rc = updateResponseFileWithLang();

    // Determine if user specified port number is valid (i.e. not it use)
    setKey("db2inst1.PORT_NUMBER");
    String portNumber = ivHelper.getResponseFileValue(this);
    // Convert the string version of the port number to an integer
    if (portNumber != null && !portNumber.equals(""))
    {
      int portNumberInt = new Integer(portNumber).intValue();
      setPortNumber(portNumberInt);
      if (!ivHelper.isPortOpen(this))
      {
        setMessage(getResourceString(NLSKeys.DB2_PORT_IN_USE, portNumber));
        ivHelper.postErrorMessageToDeployerUI(this);
        rc = FAILURE;
      }
    }

    setResponseFileValue("db2inst1.SVCENAME", "db2c_" + getProperty("db2inst1.NAME").toLowerCase());

    return rc;
  }

  /**
   * Helper method to set a value in the response file.
   * 
   * @param key The attribute to be modified in the response file.
   * @param keyValue The value that you wish to set for a given attribute.
   */
  private void setResponseFileValue(String key, String keyValue)
  {
    setKey(key);
    setKeyValue(keyValue);
    ivHelper.setResponseFileValue(this);
  }

  /**
   * performInstall() - Build up the command, and run it.
   *
   * @return - SUCCESS if successful, FAILURE if exception occurred, or the
   *           command did not return successfully.
   */
  private int performInstall()
  {
    int rc = SUCCESS;

    // Extract the image tar file
    String unpackedDir = ivHelper.getUnpackedDir(this);
    String[] command = { "/bin/sh", "-c", "tar xf " + unpackedDir + ivMediaFileName + " -C " + unpackedDir };
    rc = invokeCommand(command);

    // Extract the DB2 language pack tar file
    if (SUCCESS == rc)
    {
      String[] command1 = { "/bin/sh", "-c", "tar xf " + unpackedDir + ivNlPackFileName + " -C " + unpackedDir };
      rc = invokeCommand(command1);
    }

    if (SUCCESS == rc)
    {
      setFileName("/tmp/DB29.1Install.log"); //sets the log file name
      String[] cmd = { "/bin/sh", "-c",
          unpackedDir + "exp/disk1/db2setup -r " + getResponseFileName() + " -l " + getFileName() };
      rc = invokeCommand(cmd);

      // check if warnings occurred during install
      if (RC_SUCCESS_WITH_WARNINGS == rc)
      {
        // indicate that a warning condition was encountered.
        setMessage(getResourceString(NLSKeys.INSTALL_WARNING));
        ivHelper.postWarningMessageToDeployerUI(this);

        // since warnings are not critical, reset rc to success
        // to allow install to continue
        rc = SUCCESS;
      }

      //Append the install log file
      if (ivHelper.fileExists(this))
      {
        // Append the install log
        ivHelper.logAppendFile(this);
      }
      else
      {
        //the log file was not found
        setMessage(getResourceString("LOG_FILE_NOT_FOUND", getFileName()));
        ivHelper.postWarningMessageToDeployerUI(this);

        //since the log file was not found, the command output can show some information.
        String cmdOutput = ivHelper.getLastCommandOutput(this);
        if (cmdOutput != null || !cmdOutput.equals(""))
        {
          setMessage(getResourceString(NLSKeys.CMD_OUTPUT, cmdOutput));
          ivHelper.postInformationMessageToDeployerUI(this);
        }
      }

      if (SUCCESS == rc)
      {
        rc = checkInstallationStatus();
      }

      //register the DB2 license (located in the installation image under /db2/license folder)
      if (SUCCESS == rc)
      {
        String[] registerLicCmd = { "/bin/sh", "-c",
            ivInstallDir + "/adm/db2licm -a " + unpackedDir + "exp/disk1/db2/license/" + ivLicenseFileName };
        rc = invokeCommand(registerLicCmd);
      }
    }

    //  Copy ER license and readme files to the license directory
    setFileName(ivInstallDir);
    if (ivHelper.fileExists(this))
    {
      if (callLicenseHandler() == FAILURE)
      {
        rc = FAILURE;
      }
    }
    return rc;
  }

  /**
   * Determines if the installation was successful by
   * getting the list of installed products, and checking
   * that one of them contains the install directory given
   * by the user.
   * @return int -SUCCESS if the product was installed, FAILURE otherwise.
   */
  private int checkInstallationStatus()
  {
    int rc = FAILURE;
    DB2InstalledProduct[] db2Products = DB2InstalledProduct.getInstalledProducts(this);
    if (db2Products != null)
    {
      for (int i = 0; i < db2Products.length; i++)
      {
        if (ivInstallDir.equals(db2Products[i].getInstalledLocation()))
        {
          rc = SUCCESS;
          break;
        }
      }
    }

    return rc;
  }

  /**
   * This will add the selected languages to the response file
   * @return SUCCESS if the response file was updated successfully, FAILURE otherwise.
   **/
  private int updateResponseFileWithLang()
  {
    int rc = SUCCESS;

    // Open the file for writing
    FileOutputStream fileOut = null;

    try
    {
      fileOut = new FileOutputStream(getResponseFileName(), true);
    }
    catch (FileNotFoundException e)
    {
      setMessage(getResourceString(NLSKeys.CMD_EXCEPTION, e.toString()));
      getHelper().log(this);
      return FAILURE;
    }

    // Create the PrintWriter
    PrintWriter pw = new PrintWriter(fileOut);
    pw.println("\n");

    //Add Language Attributes to response file. 
    for (int i = 0; i < ivLangs.length; i++)
    {
      if (getProperty(ivLangs[i]).equalsIgnoreCase("true"))
      {
        pw.println("LANG                          = " + ivLangs[i]);
      }
    }

    // Close the file and check for errors
    pw.close();

    if (pw.checkError())
    {
      setMessage(getResourceString(NLSKeys.RESPONSE_FILE_UPDATE_FAIL));
      getHelper().log(this);
      rc = FAILURE;
    }

    return rc;
  }

  /**
   * invokeCommand(String [] command) - invoke the command to be run.
   *
   * @param command -
   *            command to be invoked
   * @return SUCCESS if successful, otherwise return FAILURE.
   */
  private int invokeCommand(String[] command)
  {
    int rc = SUCCESS;
    setCommandArray(command);
    setMessage(getResourceString("COMMAND_INVOKED", getHelper().getStringFromCommandArray(this)));
    ivHelper.log(this);

    // run the command
    rc = ivHelper.launchAndWait(this);

    if (SUCCESS == rc)
      setMessage(getResourceString(NLSKeys.CMD_SUCCESS));
    else
      setMessage(getResourceString(NLSKeys.CMD_FAIL, String.valueOf(rc)));

    ivHelper.log(this);
    return rc;
  }

  /**
   * Helper method to return property values from the Property object
   *
   * @param Property
   *            Key
   * @return Property Value
   */
  private String getProperty(String key)
  {
    setVariableName(key);
    return (ivHelper.getIbmNsiPropValue(this));
  }

  /**
   * Method to call the LicenseHandler program that places readme files down.
   */
  private int callLicenseHandler()
  {
    LicenseHandler licenseHandler = new LicenseHandler(getJarFile(),
        null,
        ivHelper.determineLogFileName(this),
        ivInstallDir + SLASH + "license");
    return (licenseHandler.copyLicenseFiles());
  }

  /**
   * Retreives the current DAS user, if their is one already installed on the machine.
   *  
   * @return The current DAS user name, or an empty string
   */
  private String getDASUser()
  {
    String existingDASUser = null;

    DB2InstalledProduct[] db2Products = DB2InstalledProduct.getInstalledProducts(this);
    //just need to check one product to see if a DAS user already exists
    if (db2Products != null && db2Products.length > 0)
    {
      if (db2Products[0].getDB2DASUser() != null && !db2Products[0].getDB2DASUser().equals(""))
      {
        existingDASUser = db2Products[0].getDB2DASUser();
      }

    }
    return existingDASUser;
  }

  /**
   * Determines wether a DAS user already exists as an instance user in the system.
   * 
   * @return TRUE if the DAS user exists as an instance user in the system, FALSE otherwise.
   */
  private boolean isDASUserInstanceUser(String dasUser)
  {
    boolean dasUserIsInstance = false;

    DB2InstalledProduct[] db2Products = DB2InstalledProduct.getInstalledProducts(this);
    if (db2Products != null)
    {
      for (int i = 0; i < db2Products.length; i++)
      {
        if (dasUser.equals(db2Products[i].getLinuxInstanceUser()))
        {
          dasUserIsInstance = true;
          break;
        }
      }
    }

    return dasUserIsInstance;
  }

  /**
   * Register License Keys - license keys are files typically with extension .lic. These files are required to register to use
   * related additional features/funtionalilty e.g. High Availability Disaster Recovery functionality.
   * This method registers .lic files found under the <WRAPPER_ID>/src (e.g. IRU2_2DB2Express9_1Lnx/src) file structure.
   * 
   * @return SUCCESS if license keys are not found or license keys registered successfully if found
   *         FAILURE otherwise
   */

  private int registerLicenseKeys()
  {
    int returnCode = SUCCESS;
    boolean regSuccess = true;
    //store the files with .lic extension in an array
    String[] licFiles = ivHelper.directoryGrep(ivHelper.getUnpackedDir(this), ".lic");
    if (licFiles != null && licFiles.length > 0)
    {
      for (int i = 0; i < licFiles.length; i++)
      {
        //set and invoke command to register the licenses key
        String command = ivInstallDir + SLASH + "adm" + SLASH + "db2licm -a " + ivHelper.getUnpackedDir(this)
            + licFiles[i];
        returnCode = ivHelper.invokeCommandLogOutput(command);
        if (returnCode == FAILURE)
        {
          setMessage(licFiles[i] + " failed to register");
          ivHelper.postErrorMessageToDeployerUI(this);
          //if registration failes for any key, set registration success falg to false and continue registering other keys 
          regSuccess = false;
        }
      }
    }
    if (!regSuccess)
    {
      returnCode = FAILURE;
    }
    return returnCode;
  }

  /**
   * DB2 Express 9.1 main program.
   * @param  args[0] - The name of the response file
   *         args[1] - The name of the tar file 
   *
   */
  public static void main(String[] args)
  {
    // Create and initialize the object
    DB2Express9_1LnxMain main = new DB2Express9_1LnxMain();
    int rc = main.init(args);

    if (rc == SUCCESS)
      rc = main.prepareForInstall();

    // Install DB2
    if (rc == SUCCESS)
      rc = main.performInstall();

    // Register license keys to use additional features/functions e.g. High Availability Disaster Recovery (HADR) functionality
    if (rc == SUCCESS)
      rc = main.registerLicenseKeys();

    System.exit(rc);
  }
}