package com.ibm.iru.db2express9_1win;

/**
 * @(#)DB2Express9_1WinMain.java
 *
 * Licensed Materials - Property of IBM
 *
 * 5724-J10
 *
 * (C) Copyright IBM Corp. 2007
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *
 * This is sample code made available for use in accordance with terms set
 * forth in the license agreement document for the IBM Express Runtime.
 */
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.PrintWriter;
import java.lang.Integer;
import com.ibm.iru.common.LicenseHandler;
import com.ibm.iru.message.NLSKeys;
import com.ibm.jsdt.support.SupportWindowsBase;
import com.ibm.jsdt.support.SupportWindowsHelper;
import com.ibm.jsdt.support.installedproduct.DB2InstalledProduct;

/**
 * This class launches the DB2 Express Edition setup.exe.
 * 
 */
public class DB2Express9_1WinMain extends SupportWindowsBase
{
  private static final String copyright0 = "Licensed Materials - Property of IBM";

  private static final String copyright1 = "5724-J10";

  private static final String copyright2 = "(C) Copyright IBM Corp. 2007 All Rights Reserved.";

  private static final String copyright3 = "US Government Users Restricted Rights - Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.";

  private static final int RC_SUCCESS_WITH_WARNINGS = 1;

  private static final int RC_SUCCESS_NEEDS_REBOOT = 3010;

  private SupportWindowsHelper ivHelper;

  private String ivInstallDir = null;

  private String[] ivLangs = { "BR", "CN", "DE", "FR", "ES", "IT", "JP", "KR", "TW", "DK", "FI", "NO", "PL", "RU",
      "SE", "CZ" };

  private String ivDb2RegSubKey = "SOFTWARE" + SLASH + "IBM" + SLASH + "DB2" + SLASH + "InstalledCopies";

  private String ivMigratePriorVersion = null;

  /**
   * init method has input parameters: parm#0 = response file name
   */
  private int init(String[] args)
  {
    int rc = SUCCESS;
    setMainResources("com.ibm.iru.message.MessagesNLS");
    ivHelper = getWindowsHelper();

    // Ensure the right number of parameters was passed
    int no_Of_Expected_args = 1;
    if (no_Of_Expected_args == args.length)
    {
      setResponseFileName(args[0]);
      // installation directory
      setKey("FILE");
      ivInstallDir = ivHelper.trimDoubleQuotes(ivHelper.getResponseFileValue(this));
      // see if this is a migration or a new install
      setKey("MIGRATE_PRIOR_VERSIONS");
      ivMigratePriorVersion = ivHelper.trimDoubleQuotes(ivHelper.getResponseFileValue(this));
    }
    else
    {
      String[] replacementText = { "DB2Express9_1WinMain", String.valueOf(no_Of_Expected_args),
          String.valueOf(args.length) };
      setMessage(getResourceString(NLSKeys.BAD_NUMBER_PGM_ARGS, replacementText));
      ivHelper.log(this);
      rc = FAILURE;
    }

    return rc;
  }

  /**
   * Prepare the system for an install. Set/append variables values in
   * response file:
   * <li>1. Check if a DAS user name is provided. If so, check that there is
   * no DAS user already in the system. There can only be one DAS user per
   * system. If a DAS user name is specified, and a DAS user already exists in
   * the system, post a warning and continue install (ignoring specified DAS
   * user info). If there is not a DAS user in the system, specify to create a
   * DAS username and password.
   * <li>2. Update response file with languages selected by the user.
   * <li>3. If the user specifies a different port to use, then determine if
   * in fact this port is valid. If in fact it is valid then a service is
   * established with that port number and the name of that service will be
   * the instance name.
   * 
   * @return SUCCESS if the updates to the response file were successful,
   *         FAILURE otherwise.
   */
  private int prepareForInstall()
  {
    int rc = SUCCESS;

    // check if DAS user name and password is specified, if a DAS user
    // already exists in the system.
    setKey("DAS_USERNAME");
    String dasUserName = ivHelper.trimDoubleQuotes(ivHelper.getResponseFileValue(this));
    setKey("DAS_PASSWORD");
    String dasUserPass = ivHelper.trimDoubleQuotes(ivHelper.getResponseFileValue(this));
    if ((dasUserName != null && !dasUserName.equals("") && dasUserPass != null && !dasUserPass.equals(""))
        || (dasUserName.equalsIgnoreCase("LocalSystem")))
    {
      String existingDASUser = getDASUser();
      if (existingDASUser != null && !existingDASUser.equals(""))
      {
        // a DAS user already exists. Log warning indicating that the
        // DAS user name
        // and password provided will not be used.
        setMessage(getResourceString(NLSKeys.DB2_DAS_USER_ALREADY_EXISTS, existingDASUser));
        ivHelper.postWarningMessageToDeployerUI(this);
        // Update response file indicating not to create a DAS
        setResponseFileValue("CREATE_DAS", "NO");
      }
      else
      {
        setResponseFileValue("CREATE_DAS", "YES");
      }
    }

    rc = updateResponseFileWithLang();

    // Determine if user specified port number is valid (i.e. not it use)
    setKey("DB2.PORT_NUMBER");
    String portNumber = ivHelper.getResponseFileValue(this);
    if (portNumber != null && !portNumber.equals(""))
    {
      int portNumberInt = new Integer(portNumber).intValue();
      setPortNumber(portNumberInt);
      if (!ivHelper.isPortOpen(this))
      {
        setMessage(getResourceString(NLSKeys.DB2_PORT_IN_USE, portNumber));
        ivHelper.postErrorMessageToDeployerUI(this);
        rc = FAILURE;
      }
    }

    setResponseFileValue("DB2.SVCENAME", "db2c_" + getProperty("DB2.NAME").toUpperCase());

    return rc;
  }

  /**
   * Prepares the system for a migration. First it will stop DB2. Then it will
   * run the db2ckmig command. This command will verify that the databases are
   * ready for migration. Migration should not take place if the databses are
   * in a state not ready to be migrated. Database migration should be done
   * manually after this wrapper runs. If the db2ckmig command is successful,
   * then the response file should be updated to indicate migration should
   * take place.
   * 
   * @return SUCCESS if DB2 can be successfully migrated to DB2 9, FAILURE
   *         otherwise.
   */
  private int prepareForMigration()
  {
    int rc = FAILURE;

    setFileName(ivInstallDir);
    if (!ivHelper.fileExists(this))
    {
      String cmd = "cmd /C mkdir \"" + ivInstallDir + "\"";
      invokeCommand(cmd);
    }

    // stop db2
    setCommand("db2stop");
    ivHelper.executeCommand(this);

    String command = "cmd /c " + ivHelper.getUnpackedDir(this) + "db2" + SLASH + "windows" + SLASH + "utilities"
        + SLASH + "db2ckmig.exe -e -L \"" + ivInstallDir + SLASH + "db2ckmig.log\"";
    setCommandLaunchDirectory(ivHelper.getUnpackedDir(this) + "db2" + SLASH + "windows" + SLASH + "utilities" + SLASH);
    rc = invokeCommand(command);

    if (rc == 0)
    {
      // Either no databases exist or the databases can be migrated
      // successfully
      // update response file to indicate migration
      setResponseFileValue("MIGRATE_PRIOR_VERSIONS", "YES");
      setMessage(getResourceString(NLSKeys.DB2_UPGRADE_STEPS));
      ivHelper.postInformationMessageToDeployerUI(this);

      rc = SUCCESS;
    }
    else
    {
      // update response file
      setResponseFileValue("MIGRATE_PRIOR_VERSIONS", "NO");

      setMessage(getResourceString(NLSKeys.DB2_DB2CKMIG_CMD_ERROR, ivInstallDir + SLASH + "db2ckmig.log"));
      ivHelper.postErrorMessageToDeployerUI(this);

      // restart db2 (to leave the DB2 in the same state as it was found)
      setCommand("db2start");
      ivHelper.executeCommand(this);
      rc = FAILURE;
    }

    return rc;
  }

  /**
   * Helper method to set a value in the response file.
   * 
   * @param key
   *            The attribute to be modified in the response file.
   * @param keyValue
   *            The value that you wish to set for a given attribute.
   */
  private void setResponseFileValue(String key, String keyValue)
  {
    setKey(key);
    setKeyValue(keyValue);
    ivHelper.setResponseFileValue(this);
  }

  /**
   * This will add the selected languages to the response file
   * 
   * @return SUCCESS if the response file was updated successfully, FAILURE
   *         otherwise.
   */
  private int updateResponseFileWithLang()
  {
    int rc = SUCCESS;

    // Open the file for writing
    FileOutputStream fileOut = null;

    try
    {
      fileOut = new FileOutputStream(getResponseFileName(), true);
    }
    catch (FileNotFoundException e)
    {
      setMessage(getResourceString(NLSKeys.CMD_EXCEPTION, e.toString()));
      getHelper().log(this);
      return FAILURE;
    }

    // Create the PrintWriter
    PrintWriter pw = new PrintWriter(fileOut);
    pw.println("\n");

    // Add Language Attributes
    for (int i = 0; i < ivLangs.length; i++)
    {
      if (getProperty(ivLangs[i]).equalsIgnoreCase("true"))
        pw.println("LANG                          = " + ivLangs[i]);
    }

    // Close the file and check for errors
    pw.close();

    if (pw.checkError())
    {
      setMessage(getResourceString(NLSKeys.RESPONSE_FILE_UPDATE_FAIL));
      getHelper().log(this);
      rc = FAILURE;
    }
    return rc;
  }

  /**
   * Retreives the current DAS user, if their is one already installed on the
   * machine.
   * 
   * @return The current DAS user name, or an empty string
   */
  private String getDASUser()
  {
    String existingDASUser = null;
    setRegistryKey(SupportWindowsBase.HKEY_LOCAL_MACHINE);
    setRegistrySubKey(ivDb2RegSubKey);
    String[] installedCopies = ivHelper.getRegistrySubkeys(this);
    if (installedCopies != null)
    {
      for (int i = 0; i < installedCopies.length; i++)
      {
        setRegistrySubKey(ivDb2RegSubKey + SLASH + installedCopies[i] + SLASH + "DAS_PROFILES");
        String[] dasUser = ivHelper.getRegistrySubkeys(this);
        // if there is something under this subkey, it means a DAS user
        // exists
        if (dasUser != null && dasUser.length > 0)
          existingDASUser = dasUser[0];
      }
    }
    return existingDASUser;
  }

  /**
   * Invoke the install. Verify the install, by checking existence in the
   * registry
   * 
   * @return SUCCESS -if the install was successful, FAILURE otherwise
   */
  private int install()
  {
    int rc = SUCCESS;
    // if installDir already has a "\" appended to the end, don't reappend
    // it
    if (!ivInstallDir.endsWith(SLASH))
    {
      ivInstallDir = ivInstallDir + SLASH;
    }

    setFileName(ivInstallDir + "DB2install.log");
    String command = "cmd /c " + ivHelper.getUnpackedDir(this) + "setup.exe /F /U " + getResponseFileName() + " /L "
        + "\"" + getFileName() + "\"";
    rc = invokeCommand(command);

    // Append the install log file
    if (ivHelper.fileExists(this))
    {
      // Append the install log
      ivHelper.logAppendFile(this);
    }
    else
    {
      setMessage(getResourceString("LOG_FILE_NOT_FOUND", getFileName()));
      ivHelper.postErrorMessageToDeployerUI(this);

      setMessage(getResourceString(NLSKeys.CMD_OUTPUT, ivHelper.getLastCommandOutput(this)));
      ivHelper.log(this);
    }

    // If there are errors, attempt to append the db2wi.log file
    if (rc != SUCCESS)
    {
      setFileName(ivInstallDir + "db2wi.log");
      if (ivHelper.fileExists(this))
      {
        // Append the install log
        ivHelper.logAppendFile(this);
      }
      else
      {
        setMessage(getResourceString("LOG_FILE_NOT_FOUND", getFileName()));
        ivHelper.log(this);
      }
    }

    // check if warnings occurred during install OR if a reboot is
    // necessary.
    if (RC_SUCCESS_WITH_WARNINGS == rc)
    {
      // indicate that a warning condition was encountered.
      setMessage(getResourceString(NLSKeys.INSTALL_WARNING));
      ivHelper.postWarningMessageToDeployerUI(this);
      // since warnings are not critical, reset rc to success to allow
      // install to continue
      rc = SUCCESS;
    }
    else if (rc == RC_SUCCESS_NEEDS_REBOOT)
    {
      // indicate that a reboot is needed.
      setMessage(getResourceString(NLSKeys.REBOOT_REQUIRED));
      ivHelper.postWarningMessageToDeployerUI(this);
      rc = SUCCESS;
    }

    if (SUCCESS == rc)
    {
      rc = checkInstallationStatus();
    }

    if (SUCCESS == rc)
    {
      if (ivInstallDir.endsWith(SLASH))
      {
        ivInstallDir = ivInstallDir.substring(0, ivInstallDir.lastIndexOf(SLASH));
      }
      DB2InstalledProduct db2Product = DB2InstalledProduct.getInstalledProduct(ivInstallDir);
      if (db2Product != null)
      {
        setKey("DB2_COPY_NAME");
        rc = db2Product.setDB2CopyDefault(ivHelper.getResponseFileValue(this));
      }
    }

    // Copy ER license and readme files to the license directory
    setFileName(ivInstallDir);
    if (ivHelper.fileExists(this))
    {
      if (callLicenseHandler() == FAILURE)
      {
        rc = FAILURE;
      }
    }

    return rc;
  }

  /**
   * This method checks the Windows registry for the DB2 Copy Name specified
   * by the user. If found, the installation was successful. The registry
   * location of the DB2 Copy name is as follows:
   * HKLM\SOFTWARE\IBM\DB2\InstalledCopies\<Copy Name>
   * 
   * @return SUCCESS if DB2 with the copy name specified is installed, FAILURE
   *         otherwise.
   */
  private int checkInstallationStatus()
  {
    int rc = FAILURE;

    // find the DB2 Copy Name given to this installation of DB2
    setKey("DB2_COPY_NAME");
    String db2CopyName = ivHelper.trimDoubleQuotes(ivHelper.getResponseFileValue(this));

    setRegistryKey(SupportWindowsBase.HKEY_LOCAL_MACHINE);
    setRegistrySubKey(ivDb2RegSubKey + SLASH + db2CopyName);

    if (getWindowsHelper().doesRegKeyExist(this))
    {
      rc = SUCCESS;
    }

    return rc;
  }

  /**
   * Invoke the command.
   * 
   * @return SUCCESS if the command was invoked successfully, FAILURE
   *         otherwise.
   */
  private int invokeCommand(String command)
  {
    int rc = SUCCESS;
    setMessage(getResourceString("COMMAND_INVOKED", command));
    ivHelper.log(this);

    setCommand(command);
    String output = ivHelper.getSystemCommandOutput(this);
    setMessage(output);
    ivHelper.log(this);
    // get the return code from the install
    rc = ivHelper.getLastCommandReturnCode(this);

    if (SUCCESS == rc)
    {
      setMessage(getResourceString(NLSKeys.CMD_SUCCESS));
    }
    else
    {
      // if rc = 1, then the command suceeded with warning messages.
      if (RC_SUCCESS_WITH_WARNINGS != rc)
        setMessage(getResourceString(NLSKeys.CMD_FAIL, String.valueOf(rc)));
    }
    ivHelper.log(this);

    return rc;
  }

  /**
   * Helper method to return property values from the Property object
   * 
   * @param Property
   *            Key
   * @return Property Value
   */
  private String getProperty(String key)
  {
    setVariableName(key);
    return (ivHelper.getIbmNsiPropValue(this));
  }

  /**
   * Method to call the LicenseHandler program to copy ER license and readme
   * files to the license directory.
   */
  private int callLicenseHandler()
  {
    LicenseHandler licenseHandler = new LicenseHandler(getJarFile(),
        null,
        getHelper().determineLogFileName(this),
        ivInstallDir + SLASH + "license");
    return (licenseHandler.copyLicenseFiles());
  }

  /**
   * Register License Keys - license keys are files typically with extension .lic. These files are required to register to use
   * related additional features/funtionalilty e.g. High Availability Disaster Recovery functionality.
   * This method registers .lic files found under the <WRAPPER_ID>/src (e.g. IRU2_2DB2Express9_1Win/src) file structure.
   * 
   * @return SUCCESS if license keys are not found or license keys registered successfully if found
   *         FAILURE otherwise
   */

  private int registerLicenseKeys()
  {
    int returnCode = SUCCESS;
    boolean regSuccess = true;

    // store the files with .lic extension in an Array
    String[] licFiles = ivHelper.directoryGrep(ivHelper.getUnpackedDir(this), ".lic");
    if (licFiles != null && licFiles.length > 0)
    {
      setPath(ivInstallDir);
      ivInstallDir = ivHelper.getWindowsShortPath(this);
      for (int i = 0; i < licFiles.length; i++)
      {
        //set and invoke command to register the licenses key
        String command = ivInstallDir + SLASH + "bin" + SLASH + "db2licm -a " + ivHelper.getUnpackedDir(this)
            + licFiles[i];
        returnCode = ivHelper.invokeCommandLogOutput(command);
        if (returnCode == FAILURE)
        {
          setMessage(licFiles[i] + " failed to register");
          ivHelper.postErrorMessageToDeployerUI(this);
          //if registration failes for any key, set registration success falg to false and continue registering other keys 
          regSuccess = false;
        }
      }
    }
    if (!regSuccess)
    {
      returnCode = FAILURE;
    }
    return returnCode;
  }

  /**
   * The entry point for the DB2 Express main program.
   */
  public static void main(String[] args)
  {
    // Create and initialize the object
    DB2Express9_1WinMain main = new DB2Express9_1WinMain();
    int rc = main.init(args);

    // Do the install
    if (rc == SUCCESS)
    {
      if (main.ivMigratePriorVersion != null && main.ivMigratePriorVersion.equalsIgnoreCase("yes"))
        rc = main.prepareForMigration();
      else
        rc = main.prepareForInstall();

      if (rc == SUCCESS)
        rc = main.install();

      // Register license keys for addition features/functions e.g. High Availability Disaster Recovery (HADR) functionality
      if (rc == SUCCESS)
        rc = main.registerLicenseKeys();
    }

    System.exit(rc);
  }
}
